@@ -21,14 +21,19 @@
 #define SCROLL_PAGE	(2 * SCR_FIELDY)
 
 
+int effectiveGameStatus();
+
 void DrawMainMenuExt(int, boolean);
 void DrawAndFadeInMainMenu(int);
 void DrawMainMenu(void);
 void DrawHallOfFame(int);
 
+void RedrawSetupScreenAfterFullscreenToggle();
+
 void HandleTitleScreen(int, int, int, int, int);
 void HandleMainMenu(int, int, int, int, int);
-void HandleChooseLevel(int, int, int, int, int);
+void HandleChooseLevelSet(int, int, int, int, int);
+void HandleChooseLevelNr(int, int, int, int, int);
 void HandleHallOfFame(int, int, int, int, int);
 void HandleInfoScreen(int, int, int, int, int);
 void HandleSetupScreen(int, int, int, int, int);
